// Date and time functions using a DS1307 RTC connected via I2C and Wire lib

#include <Wire.h>
#include <SPI.h>
#include <RTClib.h>
#include "DHT.h"
#include <RTC_DS1307.h>
#include <LiquidCrystal.h>
#include <BMP085.h>

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
DHT dht;
BMP085 dps = BMP085();

long Temperature = 0, Pressure = 0, Altitude = 0, light = 0, tmp;
float temperature_dht = 0, humidity = 0;
int mode=0, button=10, timer=0, mq_gas;
char  led = 9;
String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete
RTC_DS1307 RTC;

byte Cel[8] = {
  0b01000,
  0b10100,
  0b01000,
  0b00110,
  0b01001,
  0b01000,
  0b01001,
  0b00110
};

void setup () {
  pinMode(button, INPUT); 
  lcd.begin(16, 2);
  lcd.print("Loading..");
  Serial.begin(57600);
  Wire.begin();
  RTC.begin();
  pinMode(led, OUTPUT);
  dps.init(); 
  dps.setMode(MODE_ULTRA_HIGHRES);
  showall();

  lcd.setCursor(0, 1);
  lcd.println("Registers dump");
  Serial.println("Registers dump");
  Serial.println("==========================================================");
  dumpRegisters();
  lcd.setCursor(0, 1);
  lcd.println("Calibration data");
  Serial.println("Calibration data");
  Serial.println("==========================================================");
  dps.dumpCalData();
  dht.setup(8);
  DateTime now = RTC.now();
  DateTime compiled = DateTime(__DATE__, __TIME__);
  if (now.unixtime() < compiled.unixtime()) {
    
      Serial.println("RTC is older than compile time! Updating");
      lcd.setCursor(0, 1);
      lcd.print("RTC is older than compile time! Updating");
      delay(2000);
      // following line sets the RTC to the date & time this sketch was compiled
      RTC.adjust(DateTime(__DATE__, __TIME__));
    

  }
  lcd.setCursor(0, 1);
  lcd.createChar(1, Cel);
  lcd.print("OK");
  Serial.println("End calibration data");
  delay(20);
  lcd.clear();
}

void loop () {
  DateTime now = RTC.now();
  lcd.setCursor(0, 0);
  lcd.print(now.year(), DEC);
  lcd.print('/');
  print_zer(now.month(), 10);
  lcd.print(now.month(), DEC);
  lcd.print('/');
  print_zer(now.day(), 10);
  lcd.print(now.day(), DEC);
  lcd.setCursor(11, 0);
  print_zer(now.hour(), 10);
  lcd.print(now.hour(), DEC);
  lcd.print(':');
  print_zer(now.minute(), 10);
  lcd.print(now.minute(), DEC);
  /*
    lcd.print(':');
   if(now.second()<10)
   lcd.print('0');
   lcd.print(now.second(), DEC);
   
   lcd.setCursor(0, 1);
   lcd.print(dht.getMinimumSamplingPeriod(),DEC);
   delay(1000);
   */

  timer++;
  if(timer>30){
    timer = 0;
    lcd.setCursor(10, 0);
    lcd.write(' ');
    mode++;
  }
  if(timer%15==0){ 
    humidity = dht.getHumidity();
    temperature_dht = dht.getTemperature();
  }
  if(digitalRead(button))
  {
    lcd.setCursor(10, 0);
    lcd.write(-1);
    if(timer < -1 && timer >= -480)
      mode++;
    if(timer > 0 || timer >= -480)
      timer = -500;   
    if(timer < -480 && timer >= -495)
    {
      lcd.setCursor(10, 0);
      lcd.write(' ');
      timer = 0;
    }
  }

  if(mode>=3)
    mode=0;

  dps.getTemperature(&Temperature); 
  dps.getPressure(&Pressure);
  dps.getAltitude(&Altitude);
  mq_gas=analogRead(3);

  lcd.setCursor(0, 1);
  switch(mode) {
  case 0:
    lcd.print("Alt:");
    lcd.print(Altitude/100,DEC);
    lcd.write(' ');
    lcd.setCursor(6, 1);
    lcd.print("Pr:");
    lcd.println(Pressure/133.3,DEC);
    break;
  case 1:
    lcd.print("T1:");
    print_zer(temperature_dht, 10);
    lcd.print(temperature_dht, 1);
    if(temperature_dht>-10)
      lcd.write(223);
    lcd.print(" H:");
    lcd.println(humidity, 1);    
    lcd.setCursor(15, 1);
    lcd.write(37);
    lcd.print(" ");
    break;
  case 2:
    lcd.print("T2:");
    print_zer(Temperature, 10);
    lcd.print(Temperature*0.1, 1);
    if(temperature_dht>-10)
      lcd.write(223);
    lcd.print(' ');
    lcd.print("G:");
    print_zer(light, 1000);
    lcd.print(light,DEC);
    lcd.print(' ');
    break;
  }
  SerialReadString();
  if(now.hour() > 6 && now.hour() < 21)
  { 
    light = 255;
  }
  if(now.hour() < 6 && now.hour() > 21)
  {
    light = 0; 
  } 
  if(now.hour() == 6)
  {
    tmp = now.minute()*100+((now.second()*100)/59);
    light = ((tmp * 255)/6000);
  }
  if(now.hour() == 21)
  {
    tmp = now.minute()*100+((now.second()*100)/59);
    light = 255-((tmp * 255)/6000);
  }
  analogWrite(led, light);
  /*
  Serial.print(int(now.minute()));
  Serial.print("*100+");
  Serial.print(int(now.second()));
  Serial.print("*255/5959=");
  tmp = now.minute()*100+((now.second()*100)/59);
  Serial.println((tmp*255)/6000);
  Serial.println(tmp*255);
 */
   Serial.println("*1#");
   Serial.print(now.year(), DEC);
   Serial.print('/');
   Serial.print(now.month(), DEC);
   Serial.print('/');
   Serial.print(now.day(), DEC);
   Serial.print(' ');
   Serial.print(now.hour(), DEC);
   Serial.print(':');
   Serial.print(now.minute(), DEC);
   Serial.print(':');
   Serial.println(now.second(), DEC);
   Serial.print("unixtime:");
   Serial.println(now.unixtime());
   Serial.print("Temp(C):");
   Serial.print(Temperature);
   Serial.print("Alt(cm):");
   Serial.print(Altitude);
   Serial.print("Pressure(Pa):");
   Serial.println(Pressure);
   Serial.print("humidity:");
   Serial.println(humidity, 1);
   Serial.print("temperature_dht:");
   Serial.println(temperature_dht, 1);
   Serial.print("mq9_gas:");
   Serial.println(mq_gas,DEC);
  Serial.println("#");
}

void showall(void) { 
  Serial.println("Current BMP085 settings");
  Serial.println("==========================================================");
  Serial.print("device address                  = 0x");
  Serial.println(dps.getDevAddr(), HEX);
  Serial.print("Mode                            = ");
  switch (dps.getMode()) {
  case MODE_ULTRA_LOW_POWER: 
    Serial.println("MODE_ULTRA_LOW_POWER");
    break;
  case MODE_STANDARD: 
    Serial.println("MODE_STANDARD");
    break;    
  case MODE_HIGHRES: 
    Serial.println("MODE_HIGHRES");
    break;    
  case MODE_ULTRA_HIGHRES:     
    Serial.println("MODE_ULTRA_HIGHRES");
    break; 
  }  
}

void dumpRegisters() {
  byte ValidRegisterAddr[]={
    0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,0xF6,0xF7,0xF8,0xF9  }; 
  byte _b, i, totregisters = sizeof(ValidRegisterAddr);
  Serial.println("---dump start---");
  Serial.println("Register address|Register data");
  Serial.println("Reg.address(hex,dec) Reg.data(bin,hex,dec)");
  for (i=0;i<totregisters;i++){    
    Serial.print("0x");
    Serial.print(ValidRegisterAddr[i], HEX);
    Serial.print(",");
    Serial.print(ValidRegisterAddr[i], DEC);
    Serial.print(",");
    dps.readmem(ValidRegisterAddr[i], 1, &_b);
    Serial.print("b");
    print_bits(_b);
    Serial.print(",0x");
    Serial.print(_b,HEX);
    Serial.print(",");
    Serial.println(_b,DEC);  
  }
  Serial.println("---dump end---");
}

void print_bits(byte val){
  int i;
  for(i=7; i>=0; i--) 
    Serial.print(val >> i & 1, BIN);
}

void print_zer(int val, int val2)
{
  int i = 0, nzero = 0;
  if(val<0) {
    val=(val*-1)/10;
    nzero=1;
  }
  if(val == 0)
    val = 9;
  while(val < val2)
  {
    if(nzero==0)
      lcd.print("0");
    if(nzero==1)
      lcd.print(" ");
    val2=val2/10;
  }
}
void SerialReadString()
{
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read(); 
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n') {
      stringComplete = true;
    } 
  }
}

